@@ -4,6 +4,7 @@ const Clutter = imports.gi.Clutter;
 const Gtk = imports.gi.Gtk;
 const St = imports.gi.St;
 const Lang = imports.lang;
+const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const Tweener = imports.ui.tweener;
 const Main = imports.ui.main;
@@ -14,9 +15,31 @@ const Params = imports.misc.params;
 const SCALE_ANIMATION_TIME = 0.25;
 // Time to animate to original position on cancel
 const SNAP_BACK_ANIMATION_TIME = 0.25;
+// Time to animate to original position on success
+const REVERT_ANIMATION_TIME = 0.75;
+
+const DragMotionResult = {
+    NO_DROP:   0,
+    COPY_DROP: 1,
+    MOVE_DROP: 2,
+    CONTINUE:  3
+};
+
+const DRAG_CURSOR_MAP = {
+    0: Shell.Cursor.DND_UNSUPPORTED_TARGET,
+    1: Shell.Cursor.DND_COPY,
+    2: Shell.Cursor.DND_MOVE
+};
+
+const DragDropResult = {
+    FAILURE:  0,
+    SUCCESS:  1,
+    CONTINUE: 2
+};
 
 let eventHandlerActor = null;
 let currentDraggable = null;
+let dragMonitors = [];
 
 function _getEventHandlerActor() {
     if (!eventHandlerActor) {
@@ -34,6 +57,18 @@ function _getEventHandlerActor() {
     return eventHandlerActor;
 }
 
+function addDragMonitor(monitor) {
+    dragMonitors.push(monitor);
+}
+
+function removeMonitor(monitor) {
+    for (let i = 0; i < dragMonitors.length; i++)
+        if (dragMonitors[i] == monitor) {
+            dragMonitors.splice(i, 1);
+            return;
+        }
+}
+
 function _Draggable(actor, params) {
     this._init(actor, params);
 }
@@ -41,6 +76,7 @@ function _Draggable(actor, params) {
 _Draggable.prototype = {
     _init : function(actor, params) {
         params = Params.parse(params, { manualMode: false,
+                                        restoreOnSuccess: false,
                                         dragActorMaxSize: undefined,
                                         dragActorOpacity: undefined });
 
@@ -50,16 +86,22 @@ _Draggable.prototype = {
                                Lang.bind(this, this._onButtonPress));
 
         this.actor.connect('destroy', Lang.bind(this, function() {
+            this._actorDestroyed = true;
+            if (this._dragInProgress)
+                this._cancelDrag(global.get_current_time());
             this.disconnectAll();
         }));
         this._onEventId = null;
 
+        this._restoreOnSuccess = params.restoreOnSuccess;
         this._dragActorMaxSize = params.dragActorMaxSize;
         this._dragActorOpacity = params.dragActorOpacity;
 
         this._buttonDown = false; // The mouse button has been pressed and has not yet been released.
         this._dragInProgress = false; // The drag has been started, and has not been dropped or cancelled yet.
-        this._snapBackInProgress = false; // The drag has been cancelled and the item is in the process of snapping back.
+        this._animationInProgress = false; // The drag is over and the item is in the process of animating to its original position (snapping back or reverting).
+
+        this._eventsGrabbed = false;
     },
 
     _onButtonPress : function (actor, event) {
@@ -107,13 +149,19 @@ _Draggable.prototype = {
     },
 
     _grabEvents: function() {
-        Clutter.grab_pointer(_getEventHandlerActor());
-        Clutter.grab_keyboard(_getEventHandlerActor());
+        if (!this._eventsGrabbed) {
+            Clutter.grab_pointer(_getEventHandlerActor());
+            Clutter.grab_keyboard(_getEventHandlerActor());
+            this._eventsGrabbed = true;
+        }
     },
 
     _ungrabEvents: function() {
-        Clutter.ungrab_pointer();
-        Clutter.ungrab_keyboard();
+        if (this._eventsGrabbed) {
+            Clutter.ungrab_pointer();
+            Clutter.ungrab_keyboard();
+            this._eventsGrabbed = false;
+        }
     },
 
     _onEvent: function(actor, event) {
@@ -125,7 +173,7 @@ _Draggable.prototype = {
             this._buttonDown = false;
             if (this._dragInProgress) {
                 return this._dragActorDropped(event);
-            } else if (this._dragActor != null && !this._snapBackInProgress) {
+            } else if (this._dragActor != null && !this._animationInProgress) {
                 // Drag must have been cancelled with Esc.
                 this._dragComplete();
                 return true;
@@ -173,6 +221,7 @@ _Draggable.prototype = {
         if (this._onEventId)
             this._ungrabActor();
         this._grabEvents();
+        global.set_cursor(Shell.Cursor.DND_IN_DRAG);
 
         this._dragX = this._dragStartX = stageX;
         this._dragY = this._dragStartY = stageY;
@@ -226,6 +275,7 @@ _Draggable.prototype = {
 
         this._dragActor.reparent(this.actor.get_stage());
         this._dragActor.raise_top();
+        Shell.util_set_hidden_from_pick(this._dragActor, true);
 
         this._dragOrigOpacity = this._dragActor.opacity;
         if (this._dragActorOpacity != undefined)
@@ -290,51 +340,101 @@ _Draggable.prototype = {
         if (this._dragActor) {
             this._dragActor.set_position(stageX + this._dragOffsetX,
                                          stageY + this._dragOffsetY);
-            // Because we want to find out what other actor is located at the current position of this._dragActor,
-            // we have to temporarily hide this._dragActor.
-            this._dragActor.hide();
+
             let target = this._dragActor.get_stage().get_actor_at_pos(Clutter.PickMode.ALL,
                                                                       stageX, stageY);
-            this._dragActor.show();
+
+            // We call observers only once per motion with the innermost
+            // target actor. If necessary, the observer can walk the
+            // parent itself.
+            let dragEvent = {
+                x: stageX,
+                y: stageY,
+                dragActor: this._dragActor,
+                source: this.actor._delegate,
+                targetActor: target
+            };
+            for (let i = 0; i < dragMonitors.length; i++) {
+                let motionFunc = dragMonitors[i].dragMotion;
+                if (motionFunc) {
+                    let result = motionFunc(dragEvent);
+                    if (result != DragMotionResult.CONTINUE) {
+                        global.set_cursor(DRAG_CURSOR_MAP[result]);
+                        return true;
+                    }
+                }
+            }
             while (target) {
                 if (target._delegate && target._delegate.handleDragOver) {
-                    let [targX, targY] = target.get_transformed_position();
+                    let [r, targX, targY] = target.transform_stage_point(stageX, stageY);
                     // We currently loop through all parents on drag-over even if one of the children has handled it.
                     // We can check the return value of the function and break the loop if it's true if we don't want
                     // to continue checking the parents.
-                    target._delegate.handleDragOver(this.actor._delegate, this._dragActor,
-                                                    (stageX - targX) / target.scale_x,
-                                                    (stageY - targY) / target.scale_y,
-                                                    event.get_time());
+                    let result = target._delegate.handleDragOver(this.actor._delegate,
+                                                                 this._dragActor,
+                                                                 targX,
+                                                                 targY,
+                                                                 event.get_time());
+                    if (result != DragMotionResult.CONTINUE) {
+                        global.set_cursor(DRAG_CURSOR_MAP[result]);
+                        return true;
+                    }
                 }
                 target = target.get_parent();
             }
+            global.set_cursor(Shell.Cursor.DND_IN_DRAG);
         }
 
         return true;
     },
 
     _dragActorDropped: function(event) {
-        // Find a drop target. Because we want to find out what other actor is located at
-        // the current position of this._dragActor, we have to temporarily hide this._dragActor.
-        this._dragActor.hide();
         let [dropX, dropY] = event.get_coords();
         let target = this._dragActor.get_stage().get_actor_at_pos(Clutter.PickMode.ALL,
                                                                   dropX, dropY);
-        this._dragActor.show();
+
+        // We call observers only once per motion with the innermost
+        // target actor. If necessary, the observer can walk the
+        // parent itself.
+        let dropEvent = {
+            dropActor: this._dragActor,
+            targetActor: target,
+            clutterEvent: event
+        };
+        for (let i = 0; i < dragMonitors.length; i++) {
+            let dropFunc = dragMonitors[i].dragDrop;
+            if (dropFunc)
+                switch (dropFunc(dropEvent)) {
+                    case DragDropResult.FAILURE:
+                    case DragDropResult.SUCCESS:
+                        return true;
+                    case DragDropResult.CONTINUE:
+                        continue;
+                }
+        }
+
         while (target) {
             if (target._delegate && target._delegate.acceptDrop) {
-                let [targX, targY] = target.get_transformed_position();
-                if (target._delegate.acceptDrop(this.actor._delegate, this._dragActor,
-                                                (dropX - targX) / target.scale_x,
-                                                (dropY - targY) / target.scale_y,
+                let [r, targX, targY] = target.transform_stage_point(dropX, dropY);
+                if (target._delegate.acceptDrop(this.actor._delegate,
+                                                this._dragActor,
+                                                targX,
+                                                targY,
                                                 event.get_time())) {
+                    if (this._actorDestroyed)
+                        return true;
                     // If it accepted the drop without taking the actor,
-                    // destroy it.
-                    if (this._dragActor.get_parent() == this._dragActor.get_stage())
-                        this._dragActor.destroy();
+                    // handle it ourselves.
+                    if (this._dragActor.get_parent() == this._dragActor.get_stage()) {
+                        if (this._restoreOnSuccess) {
+                            this._restoreDragActor(event.get_time());
+                            return true;
+                        } else
+                            this._dragActor.destroy();
+                    }
 
                     this._dragInProgress = false;
+                    global.unset_cursor();
                     this.emit('drag-end', event.get_time(), true);
                     this._dragComplete();
                     return true;
@@ -348,34 +448,85 @@ _Draggable.prototype = {
         return true;
     },
 
-    _cancelDrag: function(eventTime) {
-        this._dragInProgress = false;
-        // Snap back to the actor source if the source is still around, snap back 
-        // to the original location if the actor itself was being dragged or the
-        // source is no longer around.
-        let snapBackX = this._snapBackX;
-        let snapBackY = this._snapBackY;
+    _getRestoreLocation: function() {
+        let x, y, scale;
+
         if (this._dragActorSource && this._dragActorSource.visible) {
-            [snapBackX, snapBackY] = this._dragActorSource.get_transformed_position();
+            // Snap the clone back to its source
+            [x, y] = this._dragActorSource.get_transformed_position();
+            let [sourceScaledWidth, sourceScaledHeight] = this._dragActorSource.get_transformed_size();
+            scale = this._dragActor.width / sourceScaledWidth;
+        } else if (this._dragOrigParent) {
+            // Snap the actor back to its original position within
+            // its parent, adjusting for the fact that the parent
+            // may have been moved or scaled
+            let [parentX, parentY] = this._dragOrigParent.get_transformed_position();
+            x = parentX + this._dragOrigParent.scale_x * this._dragOrigX;
+            y = parentY + this._dragOrigParent.scale_y * this._dragOrigY;
+
+            let [parentWidth, parentHeight] = this._dragOrigParent.get_size();
+            let [parentScaledWidth, parentScaledHeight] = this._dragOrigParent.get_transformed_size();
+            let parentScale = parentScaledWidth / parentWidth;
+            scale = this._dragOrigScale * parentScale;
+        } else {
+            // Snap back actor to its original stage position
+            x = this._snapBackX;
+            y = this._snapBackY;
+            scale = this._snapBackScale;
         }
 
-        this._snapBackInProgress = true;
+        return [x, y, scale];
+    },
+
+    _cancelDrag: function(eventTime) {
+        this._dragInProgress = false;
+        let [snapBackX, snapBackY, snapBackScale] = this._getRestoreLocation();
+
+        if (this._actorDestroyed) {
+            global.unset_cursor();
+            if (!this._buttonDown)
+                this._ungrabEvents();
+            this.emit('drag-end', eventTime, false);
+            return;
+        }
+
+        this._animationInProgress = true;
         // No target, so snap back
         Tweener.addTween(this._dragActor,
                          { x: snapBackX,
                            y: snapBackY,
-                           scale_x: this._snapBackScale,
-                           scale_y: this._snapBackScale,
+                           scale_x: snapBackScale,
+                           scale_y: snapBackScale,
                            opacity: this._dragOrigOpacity,
                            time: SNAP_BACK_ANIMATION_TIME,
                            transition: 'easeOutQuad',
-                           onComplete: this._onSnapBackComplete,
+                           onComplete: this._onAnimationComplete,
                            onCompleteScope: this,
                            onCompleteParams: [this._dragActor, eventTime]
                          });
     },
 
-    _onSnapBackComplete : function (dragActor, eventTime) {
+    _restoreDragActor: function(eventTime) {
+        this._dragInProgress = false;
+        [restoreX, restoreY, restoreScale] = this._getRestoreLocation();
+
+        // fade the actor back in at its original location
+        this._dragActor.set_position(restoreX, restoreY);
+        this._dragActor.set_scale(restoreScale, restoreScale);
+        this._dragActor.opacity = 0;
+
+        this._animationInProgress = true;
+        Tweener.addTween(this._dragActor,
+                         { opacity: this._dragOrigOpacity,
+                           time: REVERT_ANIMATION_TIME,
+                           transition: 'easeOutQuad',
+                           onComplete: this._onAnimationComplete,
+                           onCompleteScope: this,
+                           onCompleteParams: [this._dragActor, eventTime]
+                         });
+    },
+
+    _onAnimationComplete : function (dragActor, eventTime) {
         if (this._dragOrigParent) {
             dragActor.reparent(this._dragOrigParent);
             dragActor.set_scale(this._dragOrigScale, this._dragOrigScale);
@@ -383,14 +534,17 @@ _Draggable.prototype = {
         } else {
             dragActor.destroy();
         }
+        global.unset_cursor();
         this.emit('drag-end', eventTime, false);
 
-        this._snapBackInProgress = false;
+        this._animationInProgress = false;
         if (!this._buttonDown)
             this._dragComplete();
     },
 
     _dragComplete: function() {
+        Shell.util_set_hidden_from_pick(this._dragActor, false);
+
         this._dragActor = undefined;
         currentDraggable = null;
         this._ungrabEvents();
