@@ -663,11 +663,11 @@ LookingGlass.prototype = {
                                         style_class: 'lg-dialog',
                                         vertical: true,
                                         visible: false });
+        this.actor.connect('key-press-event', Lang.bind(this, this._globalKeyPressEvent));
 
-        let gconf = GConf.Client.get_default();
-        gconf.add_dir('/desktop/gnome/interface', GConf.ClientPreloadType.PRELOAD_NONE);
-        gconf.notify_add('/desktop/gnome/interface/monospace_font_name',
-                         Lang.bind(this, this._updateFont));
+        this._interfaceSettings = new Gio.Settings({ schema: 'org.gnome.desktop.interface' });
+        this._interfaceSettings.connect('changed::monospace-font-name',
+                                        Lang.bind(this, this._updateFont));
         this._updateFont();
 
         Main.uiGroup.add_actor(this.actor);
@@ -778,8 +778,7 @@ LookingGlass.prototype = {
     },
 
     _updateFont: function() {
-        let gconf = GConf.Client.get_default();
-        let fontName = gconf.get_string('/desktop/gnome/interface/monospace_font_name');
+        let fontName = this._interfaceSettings.get_string('monospace-font-name');
         // This is mishandled by the scanner - should by Pango.FontDescription_from_string(fontName);
         // https://bugzilla.gnome.org/show_bug.cgi?id=595889
         let fontDesc = Pango.font_description_from_string(fontName);
@@ -925,20 +924,15 @@ LookingGlass.prototype = {
         if (this._open)
             return;
 
-        if (!Main.pushModal(this.actor))
+        if (!Main.pushModal(this._entry))
             return;
 
-        this._keyPressEventId = global.stage.connect('key-press-event',
-            Lang.bind(this, this._globalKeyPressEvent));
-
         this.actor.show();
         this.actor.lower(Main.chrome.actor);
         this._open = true;
 
         Tweener.removeTweens(this.actor);
 
-        global.stage.set_key_focus(this._entry);
-
         // We inverse compensate for the slow-down so you can change the factor
         // through LookingGlass without long waits.
         Tweener.addTween(this.actor, { time: 0.5 / St.get_slow_down_factor(),
@@ -951,9 +945,6 @@ LookingGlass.prototype = {
         if (!this._open)
             return;
 
-        if (this._keyPressEventId)
-            global.stage.disconnect(this._keyPressEventId);
-
         this._objInspector.actor.hide();
 
         this._historyNavIndex = -1;
@@ -966,7 +957,7 @@ LookingGlass.prototype = {
             this._borderPaintTarget = null;
         }
 
-        Main.popModal(this.actor);
+        Main.popModal(this._entry);
 
         Tweener.addTween(this.actor, { time: 0.5 / St.get_slow_down_factor(),
                                        transition: 'easeOutQuad',
