@@ -37,6 +37,7 @@ const ShellDBus = imports.ui.shellDBus;
 const TelepathyClient = imports.ui.telepathyClient;
 const WindowManager = imports.ui.windowManager;
 const Magnifier = imports.ui.magnifier;
+const XdndHandler = imports.ui.xdndHandler;
 const StatusIconDispatcher = imports.ui.statusIconDispatcher;
 
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
@@ -60,6 +61,7 @@ let modalCount = 0;
 let modalActorFocusStack = [];
 let uiGroup = null;
 let magnifier = null;
+let xdndHandler = null;
 let statusIconDispatcher = null;
 let _errorLogStack = [];
 let _startDate;
@@ -107,10 +109,7 @@ function start() {
     global.stage.color = DEFAULT_BACKGROUND_COLOR;
     global.stage.no_clear_hint = true;
 
-    let themeContext = St.ThemeContext.get_for_stage (global.stage);
-    let stylesheetPath = global.datadir + '/theme/gnome-shell.css';
-    let theme = new St.Theme ({ application_stylesheet: stylesheetPath });
-    themeContext.set_theme (theme);
+    loadTheme();
 
     let shellwm = global.window_manager;
     shellwm.takeover_keybinding('panel_main_menu');
@@ -129,6 +128,7 @@ function start() {
     global.stage.add_actor(uiGroup);
 
     placesManager = new PlaceDisplay.PlacesManager();
+    xdndHandler = new XdndHandler.XdndHandler();
     overview = new Overview.Overview();
     chrome = new Chrome.Chrome();
     magnifier = new Magnifier.Magnifier();
@@ -198,6 +198,18 @@ function start() {
     }
 }
 
+/**
+ * loadTheme:
+ *
+ * Reloads the theme CSS file from the default theme.
+ */
+function loadTheme() {
+    let themeContext = St.ThemeContext.get_for_stage (global.stage);
+    let stylesheetPath = global.datadir + '/theme/gnome-shell.css';
+    let theme = new St.Theme ({ application_stylesheet: stylesheetPath });
+    themeContext.set_theme (theme);
+}
+
 /**
  * _log:
  * @category: string message type ('info', 'error')
@@ -286,47 +298,45 @@ function _removeUnusedWorkspaces() {
 // are disabled with a global grab. (When there is a global grab, then
 // all key events will be delivered to the stage, so ::captured-event
 // on the stage can be used for global keybindings.)
-//
-// We expect to need to conditionally enable just a few keybindings
-// depending on circumstance; the main hackiness here is that we are
-// assuming that keybindings have their default values; really we
-// should be asking Mutter to resolve the key into an action and then
-// base our handling based on the action.
 function _globalKeyPressHandler(actor, event) {
     if (modalCount == 0)
         return false;
-    if (event.type() != Clutter.EventType.KEY_RELEASE)
+    if (event.type() != Clutter.EventType.KEY_PRESS)
         return false;
 
     let symbol = event.get_key_symbol();
     let keyCode = event.get_key_code();
     let modifierState = Shell.get_event_state(event);
-    // Check the overview key first, this isn't a Meta.KeyBindingAction yet
-    if (symbol == Clutter.Super_L || symbol == Clutter.Super_R) {
-        // The super key is the default for triggering the overview, and should
-        // get us out of the overview when we are already in it.
-        if (overview.visible)
-            overview.hide();
 
+    let display = global.screen.get_display();
+    // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
+    let action = display.get_keybinding_action(keyCode, modifierState);
+
+    // The screenshot action should always be available (even if a
+    // modal dialog is present)
+    if (action == Meta.KeyBindingAction.COMMAND_SCREENSHOT) {
+        let gconf = GConf.Client.get_default();
+        let command = gconf.get_string('/apps/metacity/keybinding_commands/command_screenshot');
+        if (command != null && command != '') {
+            let [ok, len, args] = GLib.shell_parse_argv(command);
+            let p = new Shell.Process({'args' : args});
+            p.run();
+        }
         return true;
     }
 
-    // Whitelist some of the Metacity actions
-    let display = global.screen.get_display();
-    let activeWorkspaceIndex = global.screen.get_active_workspace_index();
+    // Other bindings are only available when the overview is up and
+    // no modal dialog is present.
+    if (!overview.visible || modalCount > 1)
+        return false;
+
+    // This isn't a Meta.KeyBindingAction yet
+    if (symbol == Clutter.Super_L || symbol == Clutter.Super_R) {
+        overview.hide();
+        return true;
+    }
 
-    // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
-    let action = display.get_keybinding_action(keyCode, modifierState);
     switch (action) {
-        case Meta.KeyBindingAction.COMMAND_SCREENSHOT:
-            let gconf = GConf.Client.get_default();
-            let command = gconf.get_string('/apps/metacity/keybinding_commands/command_screenshot');
-            if (command != null && command != '') {
-                let [ok, len, args] = GLib.shell_parse_argv(command);
-                let p = new Shell.Process({'args' : args});
-                p.run();
-            }
-            return true;
         case Meta.KeyBindingAction.WORKSPACE_LEFT:
             wm.actionMoveWorkspaceLeft();
             return true;
@@ -337,14 +347,12 @@ function _globalKeyPressHandler(actor, event) {
         case Meta.KeyBindingAction.COMMAND_2:
             getRunDialog().open();
             return true;
+        case Meta.KeyBindingAction.PANEL_MAIN_MENU:
+            overview.hide();
+            return true;
         case Meta.KeyBindingAction.SWITCH_PANELS:
-            // Only intercept this when we're in the overview, and don't
-            // intercept it if something beyond that (like, say, the
-            // ctrl-alt-tab popup!) is visible
-            if (overview.visible && modalCount == 1) {
-                ctrlAltTabManager.popup(modifierState & Clutter.ModifierType.SHIFT_MASK);
-                return true;
-            }
+            ctrlAltTabManager.popup(modifierState & Clutter.ModifierType.SHIFT_MASK);
+            return true;
     }
 
     return false;
