@@ -38,6 +38,17 @@ const State = {
     HIDING:  3
 };
 
+// Message tray has its custom Urgency enumeration. LOW, NORMAL and CRITICAL
+// urgency values map to the corresponding values for the notifications received
+// through the notification daemon. HIGH urgency value is used for chats received
+// through the Telepathy client.
+const Urgency = {
+    LOW: 0,
+    NORMAL: 1,
+    HIGH: 2,
+    CRITICAL: 3
+}
+
 function _fixMarkup(text, allowMarkup) {
     if (allowMarkup) {
         // Support &amp;, &quot;, &apos;, &lt; and &gt;, escape all other
@@ -225,7 +236,10 @@ function Notification(source, title, banner, params) {
 Notification.prototype = {
     _init: function(source, title, banner, params) {
         this.source = source;
-        this.urgent = false;
+        this.urgency = Urgency.NORMAL;
+        this.resident = false;
+        // 'transient' is a reserved keyword in JS, so we have to use an alternate variable name
+        this.isTransient = false;
         this.expanded = false;
         this._useActionIcons = false;
         this._customContent = false;
@@ -242,7 +256,6 @@ Notification.prototype = {
         this._prevFocusedWindow = null;
         this._prevKeyFocusActor = null;
 
-        this._focusWindowChangedId = 0;
         this._focusActorChangedId = 0;
         this._stageInputModeChangedId = 0;
         this._capturedEventId = 0;
@@ -257,7 +270,7 @@ Notification.prototype = {
             function (actor, event) {
                 if (!this._actionArea ||
                     !this._actionArea.contains(event.get_source()))
-                    this.emit('clicked');
+                    this._onClicked();
             }));
 
         // The first line should have the title, followed by the
@@ -484,12 +497,20 @@ Notification.prototype = {
         }
 
         this._buttonBox.add(button);
-        button.connect('clicked', Lang.bind(this, function() { this.emit('action-invoked', id); }));
+        button.connect('clicked', Lang.bind(this, this._onActionInvoked, id));
         this._updated();
     },
 
-    setUrgent: function(urgent) {
-        this.urgent = urgent;
+    setUrgency: function(urgency) {
+        this.urgency = urgency;
+    },
+
+    setResident: function(resident) {
+        this.resident = resident;
+    },
+
+    setTransient: function(isTransient) {
+        this.isTransient = isTransient;
     },
 
     setUseActionIcons: function(useIcons) {
@@ -613,29 +634,16 @@ Notification.prototype = {
         let metaDisplay = global.screen.get_display();
 
         this._prevFocusedWindow = metaDisplay.focus_window;
-        this._prevKeyFocus = global.stage.get_key_focus();
+        this._prevKeyFocusActor = global.stage.get_key_focus();
 
-        // We need to use the captured event in the overview, because we don't want to change the stage input mode to
-        // FOCUSED there. On the other hand, using the captured event doesn't work correctly in the main view because
-        // it doesn't allow focusing the windows again correctly. So we are using the FOCUSED stage input mode in the
-        // main view.
-        if (Main.overview.visible) {
-            if (!Main.pushModal(this.actor))
-                return;
-            this._capturedEventId = global.stage.connect('captured-event', Lang.bind(this, this._onCapturedEvent));
-        } else {
+        if (!Main.overview.visible)
             global.set_stage_input_mode(Shell.StageInputMode.FOCUSED);
 
-            this._focusWindowChangedId = metaDisplay.connect('notify::focus-window', Lang.bind(this, this._focusWindowChanged));
-            this._stageInputModeChangedId = global.connect('notify::stage-input-mode', Lang.bind(this, this._stageInputModeChanged));
+        // Use captured-event to notice clicks outside the notification
+        // without consuming them.
+        this._capturedEventId = global.stage.connect('captured-event', Lang.bind(this, this._onCapturedEvent));
 
-            this._keyPressId = global.stage.connect('key-press-event', Lang.bind(this, this._onKeyPress));
-        }
-
-        // We need to listen to this signal in the overview, as well as in the main view, to make the key bindings such as
-        // Alt+F2 work. When a notification has key focus, which is the case with chat notifications, all captured KEY_PRESS
-        // events have the actor with the key focus as their source. This makes it impossible to distinguish between the chat
-        // window input and the key bindings based solely on the KEY_PRESS event.
+        this._stageInputModeChangedId = global.connect('notify::stage-input-mode', Lang.bind(this, this._stageInputModeChanged));
         this._focusActorChangedId = global.stage.connect('notify::key-focus', Lang.bind(this, this._focusActorChanged));
 
         this._hasFocus = true;
@@ -643,19 +651,6 @@ Notification.prototype = {
             Main.messageTray.lock();
     },
 
-    _focusWindowChanged: function() {
-        let metaDisplay = global.screen.get_display();
-        // this._focusWindowChanged() will be called when we call
-        // global.set_stage_input_mode(Shell.StageInputMode.FOCUSED) ,
-        // however metaDisplay.focus_window will be null in that case. We only
-        // want to ungrab focus if the focus has been moved to an application
-        // window.
-        if (metaDisplay.focus_window) {
-            this._prevFocusedWindow = null;
-            this.ungrabFocus();
-        }
-    },
-
     _focusActorChanged: function() {
         let focusedActor = global.stage.get_key_focus();
         if (!focusedActor || !this.actor.contains(focusedActor)) {
@@ -665,15 +660,6 @@ Notification.prototype = {
     },
 
     _stageInputModeChanged: function() {
-        let focusedActor = global.stage.get_key_focus();
-        // TODO: We need to set this._prevFocusedWindow to null in order to
-        // get the cursor in the run dialog. However, that also means it's
-        // set to null when the application menu is activated, which defeats
-        // the point of keeping the name of the previously focused application
-        // in the panel. It'd be good to be able to distinguish between these
-        // two cases.
-        this._prevFocusedWindow = null;
-        this._prevKeyFocusActor = null;
         this.ungrabFocus();
     },
 
@@ -696,6 +682,28 @@ Notification.prototype = {
         return false;
     },
 
+    _onActionInvoked: function(actor, id) {
+        this.emit('action-invoked', id);
+        if (!this.resident) {
+            // We don't hide a resident notification when the user invokes one of its actions,
+            // because it is common for such notifications to update themselves with new
+            // information based on the action. We'd like to display the updated information
+            // in place, rather than pop-up a new notification.
+            this.emit('done-displaying');
+            this.destroy();
+        }
+    },
+
+    _onClicked: function() {
+        this.emit('clicked');
+        // We hide all types of notifications once the user clicks on them because the common
+        // outcome of clicking should be the relevant window being brought forward and the user's
+        // attention switching to the window.
+        this.emit('done-displaying');
+        if (!this.resident)
+            this.destroy();
+    },
+
     _onKeyPress: function(actor, event) {
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.Escape) {
@@ -710,10 +718,6 @@ Notification.prototype = {
             return;
 
         let metaDisplay = global.screen.get_display();
-        if (this._focusWindowChangedId > 0) {
-            metaDisplay.disconnect(this._focusWindowChangedId);
-            this._focusWindowChangedId = 0;
-        }
 
         if (this._focusActorChangedId > 0) {
             global.stage.disconnect(this._focusActorChangedId);
@@ -726,20 +730,14 @@ Notification.prototype = {
         }
 
         if (this._capturedEventId > 0) {
-            Main.popModal(this.actor);
             global.stage.disconnect(this._capturedEventId);
             this._capturedEventId = 0;
         }
 
-        if (this._keyPressId > 0) {
-            global.stage.disconnect(this._keyPressId);
-            this._keyPressId = 0;
-        }
-
         this._hasFocus = false;
         Main.messageTray.unlock();
 
-        if (this._prevFocusedWindow) {
+        if (this._prevFocusedWindow && !metaDisplay.focus_window) {
             metaDisplay.set_input_focus_window(this._prevFocusedWindow, false, global.get_current_time());
             this._prevFocusedWindow = null;
         }
@@ -781,6 +779,11 @@ Source.prototype = {
         this.title = title;
         this._iconBin = new St.Bin({ width: this.ICON_SIZE,
                                      height: this.ICON_SIZE });
+        this.isTransient = false;
+    },
+
+    setTransient: function(isTransient) {
+        this.isTransient = isTransient;
     },
 
     // Called to create a new icon actor (of size this.ICON_SIZE).
@@ -811,6 +814,7 @@ Source.prototype = {
                     this.notification = null;
                     this._notificationDestroyedId = 0;
                     this._notificationClickedId = 0;
+                    this._notificationRemoved();
                 }
             }));
 
@@ -830,8 +834,13 @@ Source.prototype = {
         this._iconBin.child = icon;
     },
 
-    // Default implementation is to do nothing, but subclass can override
+    // Default implementation is to do nothing, but subclasses can override
     _notificationClicked: function(notification) {
+    },
+
+    // Default implementation is to destroy this source, but subclasses can override
+    _notificationRemoved: function() {
+        this.destroy();
     }
 };
 Signals.addSignalMethods(Source.prototype);
@@ -905,8 +914,9 @@ MessageTray.prototype = {
         this._notificationBin.hide();
         this._notificationQueue = [];
         this._notification = null;
+        this._notificationClickedId = 0;
 
-        this._summaryBin = new St.Bin({ anchor_gravity: Clutter.Gravity.NORTH_EAST });
+        this._summaryBin = new St.Bin({ x_align: St.Align.END });
         this.actor.add_actor(this._summaryBin);
         this._summary = new St.BoxLayout({ name: 'summary-mode',
                                            reactive: true,
@@ -926,6 +936,7 @@ MessageTray.prototype = {
         this._summaryNotificationBoxPointer.actor.hide();
 
         this._summaryNotification = null;
+        this._summaryNotificationClickedId = 0;
         this._clickedSummaryItem = null;
         this._clickedSummaryItemAllocationChangedId = 0;
         this._expandedSummaryItem = null;
@@ -997,9 +1008,8 @@ MessageTray.prototype = {
         this.actor.width = primary.width;
         this._notificationBin.x = 0;
         this._notificationBin.width = primary.width;
-
-        // These work because of their anchor_gravity
-        this._summaryBin.x = primary.width;
+        this._summaryBin.x = 0;
+        this._summaryBin.width = primary.width;
     },
 
     contains: function(source) {
@@ -1033,7 +1043,15 @@ MessageTray.prototype = {
         }
 
         this._summaryItems.push(summaryItem);
-        this._newSummaryItems.push(summaryItem);
+
+        // We keep this._newSummaryItems to track any new sources that were added to the
+        // summary and show the summary with them to the user for a short period of time
+        // after notifications are done showing. However, we don't want that to happen for
+        // transient sources, which are removed after the notification is shown, but are
+        // not removed fast enough because of the callbacks to avoid the summary popping up.
+        // So we just don't add transient sources to this._newSummaryItems .
+        if (!source.isTransient)
+            this._newSummaryItems.push(summaryItem);
 
         source.connect('notify', Lang.bind(this, this._onNotify));
 
@@ -1158,13 +1176,11 @@ MessageTray.prototype = {
         } else if (this._notificationQueue.indexOf(notification) < 0) {
             notification.connect('destroy',
                                  Lang.bind(this, this.removeNotification));
-
-            if (notification.urgent)
-                this._notificationQueue.unshift(notification);
-            else
-                this._notificationQueue.push(notification);
+            this._notificationQueue.push(notification);
+            this._notificationQueue.sort(function(notification1, notification2) {
+                return (notification2.urgency - notification1.urgency);
+            });
         }
-
         this._updateState();
     },
 
@@ -1363,7 +1379,7 @@ MessageTray.prototype = {
         let notificationsPending = this._notificationQueue.length > 0;
         let notificationPinned = this._pointerInTray && !this._pointerInSummary && !this._notificationRemoved;
         let notificationExpanded = this._notificationBin.y < 0;
-        let notificationExpired = (this._notificationTimeoutId == 0 && !this._pointerInTray && !this._locked) || this._notificationRemoved;
+        let notificationExpired = (this._notificationTimeoutId == 0 && !(this._notification && this._notification.urgency == Urgency.CRITICAL) && !this._pointerInTray && !this._locked) || this._notificationRemoved;
 
         if (this._notificationState == State.HIDDEN) {
             if (notificationsPending)
@@ -1465,6 +1481,8 @@ MessageTray.prototype = {
 
     _showNotification: function() {
         this._notification = this._notificationQueue.shift();
+        this._notificationClickedId = this._notification.connect('done-displaying',
+                                                                 Lang.bind(this, this.escapeTray));
         this._notificationBin.child = this._notification.actor;
 
         this._notificationBin.opacity = 0;
@@ -1498,18 +1516,19 @@ MessageTray.prototype = {
                       onCompleteScope: this
                     });
 
-        // We auto-expand urgent notifications.
+        // We auto-expand notifications with CRITICAL urgency.
         // We call _expandNotification() again on the notifications that
         // are expanded in case they were in the process of hiding and need
         // to re-expand.
-        if (this._notification.urgent || this._notification.expanded)
+        if (this._notification.urgency == Urgency.CRITICAL || this._notification.expanded)
             // This will overwrite the y tween, but leave the opacity
             // tween, and so the onComplete will remain as well.
             this._expandNotification(true);
    },
 
     _showNotificationCompleted: function() {
-        this._updateNotificationTimeout(NOTIFICATION_TIMEOUT * 1000);
+        if (this._notification.urgency != Urgency.CRITICAL)
+            this._updateNotificationTimeout(NOTIFICATION_TIMEOUT * 1000);
     },
 
     _updateNotificationTimeout: function(timeout) {
@@ -1562,7 +1581,12 @@ MessageTray.prototype = {
         this._notificationBin.hide();
         this._notificationBin.child = null;
         this._notification.collapseCompleted();
+        this._notification.disconnect(this._notificationClickedId);
+        this._notificationClickedId = 0;
+        let notification = this._notification;
         this._notification = null;
+        if (notification.isTransient)
+            notification.destroy();
     },
 
     _expandNotification: function(autoExpanding) {
@@ -1589,7 +1613,7 @@ MessageTray.prototype = {
    },
 
     // We use this function to grab focus when the user moves the pointer
-    // to an urgent notification that was already auto-expanded.
+    // to a notification with CRITICAL urgency that was already auto-expanded.
     _ensureNotificationFocused: function() {
         this._notification.grabFocus(false);
     },
@@ -1636,7 +1660,8 @@ MessageTray.prototype = {
 
     _showSummaryNotification: function() {
         this._summaryNotification = this._clickedSummaryItem.source.notification;
-
+        this._summaryNotificationClickedId = this._summaryNotification.connect('done-displaying',
+                                                                               Lang.bind(this, this.escapeTray));
         let index = this._notificationQueue.indexOf(this._summaryNotification);
         if (index != -1)
             this._notificationQueue.splice(index, 1);
@@ -1659,8 +1684,8 @@ MessageTray.prototype = {
         this._summaryNotificationBoxPointer.actor.show();
         this._adjustNotificationBoxPointerPosition();
 
-        this._summaryNotificationState = State.SHOWNING;
-        this._summaryNotificationBoxPointer.animateAppear(Lang.bind(this, function() {
+        this._summaryNotificationState = State.SHOWING;
+        this._summaryNotificationBoxPointer.show(true, Lang.bind(this, function() {
             this._summaryNotificationState = State.SHOWN;
         }));
     },
@@ -1699,15 +1724,19 @@ MessageTray.prototype = {
 
         this._summaryNotification.ungrabFocus();
         this._summaryNotificationState = State.HIDING;
-        this._summaryNotificationBoxPointer.animateDisappear(Lang.bind(this, this._hideSummaryNotificationCompleted));
+        this._summaryNotificationBoxPointer.hide(true, Lang.bind(this, this._hideSummaryNotificationCompleted));
     },
 
     _hideSummaryNotificationCompleted: function() {
         this._summaryNotificationState = State.HIDDEN;
         this._summaryNotificationBoxPointer.bin.child = null;
         this._summaryNotification.collapseCompleted();
+        this._summaryNotification.disconnect(this._summaryNotificationClickedId);
+        this._summaryNotificationClickedId = 0;
         let summaryNotification = this._summaryNotification;
         this._summaryNotification = null;
+        if (summaryNotification.isTransient && !this._reNotifyWithSummaryNotificationAfterHide)
+            summaryNotification.destroy();
         if (this._reNotifyWithSummaryNotificationAfterHide) {
             this._onNotify(summaryNotification.source, summaryNotification);
             this._reNotifyWithSummaryNotificationAfterHide = false;
