@@ -6,6 +6,7 @@
 #include "shell-enum-types.h"
 #include "shell-perf-log.h"
 #include "shell-window-tracker.h"
+#include "shell-marshal.h"
 #include "shell-wm.h"
 #include "st.h"
 
@@ -35,7 +36,6 @@
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
 static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data);
-static void update_root_window_pixmap (ShellGlobal *global);
 
 struct _ShellGlobal {
   GObject parent;
@@ -61,8 +61,6 @@ struct _ShellGlobal {
   const char *userdatadir;
   StFocusManager *focus_manager;
 
-  /* Displays the root window; see shell_global_create_root_pixmap_actor() */
-  ClutterActor *root_pixmap;
   GdkWindow    *stage_window;
 
   gint last_change_screen_width, last_change_screen_height;
@@ -73,6 +71,8 @@ struct _ShellGlobal {
 
   /* For sound notifications */
   ca_context *sound_context;
+
+  guint32 xdnd_timestamp;
 };
 
 enum {
@@ -86,6 +86,7 @@ enum {
   PROP_STAGE,
   PROP_STAGE_INPUT_MODE,
   PROP_WINDOW_GROUP,
+  PROP_BACKGROUND_ACTOR,
   PROP_WINDOW_MANAGER,
   PROP_SETTINGS,
   PROP_DATADIR,
@@ -94,8 +95,19 @@ enum {
   PROP_FOCUS_MANAGER,
 };
 
+/* Signals */
+enum
+{
+ XDND_POSITION_CHANGED,
+ XDND_LEAVE,
+ XDND_ENTER,
+ LAST_SIGNAL
+};
+
 G_DEFINE_TYPE(ShellGlobal, shell_global, G_TYPE_OBJECT);
 
+static guint shell_global_signals [LAST_SIGNAL] = { 0 };
+
 static void
 shell_global_set_property(GObject         *object,
                           guint            prop_id,
@@ -160,6 +172,9 @@ shell_global_get_property(GObject         *object,
     case PROP_WINDOW_GROUP:
       g_value_set_object (value, meta_plugin_get_window_group (global->plugin));
       break;
+    case PROP_BACKGROUND_ACTOR:
+      g_value_set_object (value, meta_plugin_get_background_actor (global->plugin));
+      break;
     case PROP_WINDOW_MANAGER:
       g_value_set_object (value, global->wm);
       break;
@@ -217,7 +232,6 @@ shell_global_init (ShellGlobal *global)
   g_signal_connect (global->grab_notifier, "grab-notify", G_CALLBACK (grab_notify), global);
   global->gtk_grab_active = FALSE;
 
-  global->root_pixmap = NULL;
   global->stage_window = NULL;
 
   global->input_mode = SHELL_STAGE_INPUT_MODE_NORMAL;
@@ -238,6 +252,36 @@ shell_global_class_init (ShellGlobalClass *klass)
   gobject_class->get_property = shell_global_get_property;
   gobject_class->set_property = shell_global_set_property;
 
+  /* Emitted from gnome-shell-plugin.c during event handling */
+  shell_global_signals[XDND_POSITION_CHANGED] =
+      g_signal_new ("xdnd-position-changed",
+                    G_TYPE_FROM_CLASS (klass),
+                    G_SIGNAL_RUN_LAST,
+                    0,
+                    NULL, NULL,
+                    _shell_marshal_VOID__INT_INT,
+                    G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
+
+  /* Emitted from gnome-shell-plugin.c during event handling */
+  shell_global_signals[XDND_LEAVE] =
+      g_signal_new ("xdnd-leave",
+                    G_TYPE_FROM_CLASS (klass),
+                    G_SIGNAL_RUN_LAST,
+                    0,
+                    NULL, NULL,
+                    g_cclosure_marshal_VOID__VOID,
+                    G_TYPE_NONE, 0);
+
+  /* Emitted from gnome-shell-plugin.c during event handling */
+  shell_global_signals[XDND_ENTER] =
+      g_signal_new ("xdnd-enter",
+                    G_TYPE_FROM_CLASS (klass),
+                    G_SIGNAL_RUN_LAST,
+                    0,
+                    NULL, NULL,
+                    g_cclosure_marshal_VOID__VOID,
+                    G_TYPE_NONE, 0);
+
   g_object_class_install_property (gobject_class,
                                    PROP_OVERLAY_GROUP,
                                    g_param_spec_object ("overlay-group",
@@ -298,6 +342,13 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Actor holding window actors",
                                                         CLUTTER_TYPE_ACTOR,
                                                         G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_BACKGROUND_ACTOR,
+                                   g_param_spec_object ("background-actor",
+                                                        "Background Actor",
+                                                        "Actor drawing root window background",
+                                                        CLUTTER_TYPE_ACTOR,
+                                                        G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_WINDOW_MANAGER,
                                    g_param_spec_object ("window-manager",
@@ -487,6 +538,9 @@ shell_global_set_cursor (ShellGlobal *global,
     case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
       name = "dnd-none";
       break;
+    case SHELL_CURSOR_POINTING_HAND:
+      name = "hand";
+      break;
     default:
       g_return_if_reached ();
     }
@@ -506,6 +560,8 @@ shell_global_set_cursor (ShellGlobal *global,
         case SHELL_CURSOR_DND_COPY:
           cursor_type = GDK_PLUS;
           break;
+        case SHELL_CURSOR_POINTING_HAND:
+          cursor_type = GDK_HAND2;
         case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
           cursor_type = GDK_X_CURSOR;
           break;
@@ -518,7 +574,8 @@ shell_global_set_cursor (ShellGlobal *global,
     {
       ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
 
-      global->stage_window = gdk_window_foreign_new (clutter_x11_get_stage_window (stage));
+      global->stage_window = gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
+                                                                     clutter_x11_get_stage_window (stage));
     }
 
   gdk_window_set_cursor (global->stage_window, cursor);
@@ -641,11 +698,6 @@ update_screen_size (gpointer data)
   global->last_change_screen_width = width;
   global->last_change_screen_height = height;
 
-  /* update size of background actor to fix tiled backgrounds */
-  if (global->root_pixmap)
-    clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
-                            width, height);
-
   return FALSE;
 }
 
@@ -1122,75 +1174,37 @@ grab_notify (GtkWidget *widget, gboolean was_grabbed, gpointer user_data)
   shell_global_set_stage_input_mode (global, global->input_mode);
 }
 
-/*
- * Updates the global->root_pixmap actor with the root window's pixmap or fails
- * with a warning.
+/**
+ * shell_global_init_xdnd:
+ * @global: the #ShellGlobal
+ *
+ * Enables tracking of Xdnd events
  */
-static void
-update_root_window_pixmap (ShellGlobal *global)
+void shell_global_init_xdnd (ShellGlobal *global)
 {
-  Atom type;
-  int format;
-  gulong nitems;
-  gulong bytes_after;
-  guchar *data;
-  Pixmap root_pixmap_id = None;
+  long xdnd_version = 5;
 
-  if (!XGetWindowProperty (gdk_x11_get_default_xdisplay (),
-                           gdk_x11_get_default_root_xwindow (),
-                           gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID"),
-                           0, LONG_MAX,
-                           False,
-                           AnyPropertyType,
-                           &type, &format, &nitems, &bytes_after, &data) &&
-      type != None)
-  {
-     /* Got a property. */
-     if (type == XA_PIXMAP && format == 32 && nitems == 1)
-       {
-         /* Was what we expected. */
-         root_pixmap_id = *(Pixmap *)data;
-       }
-     else
-       {
-         g_warning ("Could not get the root window pixmap");
-       }
+  MetaScreen *screen = shell_global_get_screen (global);
+  Window output_window = meta_get_overlay_window (screen);
 
-     XFree(data);
-  }
+  MetaDisplay *display = meta_screen_get_display (screen);
+  Display *xdisplay = meta_display_get_xdisplay (display);
 
-  clutter_x11_texture_pixmap_set_pixmap (CLUTTER_X11_TEXTURE_PIXMAP (global->root_pixmap),
-                                         root_pixmap_id);
-}
+  ClutterStage *stage = CLUTTER_STAGE(meta_plugin_get_stage (global->plugin));
+  Window stage_win = clutter_x11_get_stage_window (stage);
 
-/*
- * Called when the X server emits a root window change event. If the event is
- * about a new pixmap, update the global->root_pixmap actor.
- */
-static GdkFilterReturn
-root_window_filter (GdkXEvent *native, GdkEvent *event, gpointer data)
-{
-  XEvent *xevent = (XEvent *)native;
+  XChangeProperty (xdisplay, stage_win, gdk_x11_get_xatom_by_name ("XdndAware"), XA_ATOM,
+                  32, PropModeReplace, (const unsigned char *)&xdnd_version, 1);
 
-  if ((xevent->type == PropertyNotify) &&
-      (xevent->xproperty.window == gdk_x11_get_default_root_xwindow ()) &&
-      (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID")))
-    update_root_window_pixmap (SHELL_GLOBAL (data));
+  XChangeProperty (xdisplay, output_window, gdk_x11_get_xatom_by_name ("XdndProxy"), XA_WINDOW,
+                  32, PropModeReplace, (const unsigned char *)&stage_win, 1);
 
-  return GDK_FILTER_CONTINUE;
-}
-
-/*
- * Called when the root window pixmap actor is destroyed.
- */
-static void
-root_pixmap_destroy (GObject *sender, gpointer data)
-{
-  ShellGlobal *global = SHELL_GLOBAL (data);
-
-  gdk_window_remove_filter (gdk_get_default_root_window (),
-                            root_window_filter, global);
-  global->root_pixmap = NULL;
+  /*
+   * XdndProxy is additionally set on the proxy window as verification that the
+   * XdndProxy property on the target window isn't a left-over
+   */
+  XChangeProperty (xdisplay, stage_win, gdk_x11_get_xatom_by_name ("XdndProxy"), XA_WINDOW,
+                  32, PropModeReplace, (const unsigned char *)&stage_win, 1);
 }
 
 /**
@@ -1241,80 +1255,6 @@ shell_global_format_time_relative_pretty (ShellGlobal *global,
    }
 }
 
-/**
- * shell_global_create_root_pixmap_actor:
- * @global: a #ShellGlobal
- *
- * Creates an actor showing the root window pixmap.
- *
- * Return value: (transfer none): a #ClutterActor with the root window pixmap.
- *               The actor is floating, hence (transfer none).
- */
-ClutterActor *
-shell_global_create_root_pixmap_actor (ShellGlobal *global)
-{
-  GdkWindow *window;
-  ClutterActor *stage;
-  ClutterColor stage_color;
-
-  /* The actor created is actually a ClutterClone of global->root_pixmap. */
-
-  if (global->root_pixmap == NULL)
-    {
-      global->root_pixmap = clutter_glx_texture_pixmap_new ();
-
-      clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
-                              global->last_change_screen_width,
-                              global->last_change_screen_height);
-
-      clutter_texture_set_repeat (CLUTTER_TEXTURE (global->root_pixmap),
-                                  TRUE, TRUE);
-
-      /* The low and medium quality filters give nearest-neighbor resizing. */
-      clutter_texture_set_filter_quality (CLUTTER_TEXTURE (global->root_pixmap),
-                                          CLUTTER_TEXTURE_QUALITY_HIGH);
-
-      /* Initialize to the stage color, since that's what will be seen
-       * in the main view if there's no actual background window.
-       */
-      stage = meta_plugin_get_stage (global->plugin);
-      clutter_stage_get_color (CLUTTER_STAGE (stage), &stage_color);
-      clutter_texture_set_from_rgb_data (CLUTTER_TEXTURE (global->root_pixmap),
-                                         /* ClutterColor has the same layout
-                                          * as one pixel of RGB(A) data.
-                                          */
-                                         (const guchar *)&stage_color, FALSE,
-                                         /* w, h, rowstride, bpp, flags */
-                                         1, 1, 3, 3, 0, NULL);
-
-      /* We can only clone an actor within a stage, so we hide the source
-       * texture then add it to the stage */
-      clutter_actor_hide (global->root_pixmap);
-      clutter_container_add_actor (CLUTTER_CONTAINER (stage),
-                                   global->root_pixmap);
-
-      /* This really should never happen; but just in case... */
-      g_signal_connect (global->root_pixmap, "destroy",
-                        G_CALLBACK (root_pixmap_destroy), global);
-
-      /* Metacity handles changes to some root window properties in its global
-       * event filter, though not _XROOTPMAP_ID. For all root window property
-       * changes, the global filter returns GDK_FILTER_CONTINUE, so our
-       * window specific filter will be called after the global one.
-       *
-       * Because Metacity is already handling root window property updates,
-       * we don't have to worry about adding the PropertyChange mask to the
-       * root window to get PropertyNotify events.
-       */
-      window = gdk_get_default_root_window ();
-      gdk_window_add_filter (window, root_window_filter, global);
-
-      update_root_window_pixmap (global);
-    }
-
-  return clutter_clone_new (global->root_pixmap);
-}
-
 /**
  * shell_global_get_monitors:
  * @global: the #ShellGlobal
@@ -1559,6 +1499,10 @@ shell_global_get_current_time (ShellGlobal *global)
   guint32 time;
   MetaDisplay *display;
 
+  /* In case we have a xdnd timestamp use it */
+  if (global->xdnd_timestamp != 0)
+    return global->xdnd_timestamp;
+
   /* meta_display_get_current_time() will return the correct time
      when handling an X or Gdk event, but will return CurrentTime
      from some Clutter event callbacks.
@@ -1826,3 +1770,67 @@ shell_global_play_theme_sound (ShellGlobal *global,
 {
   ca_context_play (global->sound_context, 0, CA_PROP_EVENT_ID, name, NULL);
 }
+
+/*
+ * Process Xdnd events
+ *
+ * We pass the position and leave events to JS via a signal
+ * where the actual drag & drop handling happens.
+ *
+ * http://www.freedesktop.org/wiki/Specifications/XDND
+ */
+gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
+                                         XEvent       *xev)
+{
+  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
+  Window output_window = meta_get_overlay_window (screen);
+  MetaDisplay *display = meta_screen_get_display (screen);
+  Display *xdisplay = meta_display_get_xdisplay (display);
+
+  ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
+  Window stage_win = clutter_x11_get_stage_window (stage);
+
+  if (xev->xany.window != output_window && xev->xany.window != stage_win)
+    return FALSE;
+
+  if (xev->xany.type == ClientMessage && xev->xclient.message_type == gdk_x11_get_xatom_by_name ("XdndPosition"))
+    {
+      XEvent xevent;
+      Window src = xev->xclient.data.l[0];
+
+      memset (&xevent, 0, sizeof(xevent));
+      xevent.xany.type = ClientMessage;
+      xevent.xany.display = xdisplay;
+      xevent.xclient.window = src;
+      xevent.xclient.message_type = gdk_x11_get_xatom_by_name ("XdndStatus");
+      xevent.xclient.format = 32;
+      xevent.xclient.data.l[0] = output_window;
+      /* flags: bit 0: will we accept the drop? bit 1: do we want more position messages */
+      xevent.xclient.data.l[1] = 2;
+      xevent.xclient.data.l[4] = None;
+
+      XSendEvent (xdisplay, src, False, 0, &xevent);
+
+      /* Store the timestamp of the xdnd position event */
+      global->xdnd_timestamp = xev->xclient.data.l[3];
+      g_signal_emit_by_name (G_OBJECT (global), "xdnd-position-changed",
+                            (int)(xev->xclient.data.l[2] >> 16), (int)(xev->xclient.data.l[2] & 0xFFFF));
+      global->xdnd_timestamp = 0;
+
+      return TRUE;
+    }
+   else if (xev->xany.type == ClientMessage && xev->xclient.message_type == gdk_x11_get_xatom_by_name ("XdndLeave"))
+    {
+      g_signal_emit_by_name (G_OBJECT (global), "xdnd-leave");
+
+      return TRUE;
+    }
+   else if (xev->xany.type == ClientMessage && xev->xclient.message_type == gdk_x11_get_xatom_by_name ("XdndEnter"))
+    {
+      g_signal_emit_by_name (G_OBJECT (global), "xdnd-enter");
+
+      return TRUE;
+    }
+
+    return FALSE;
+}
