@@ -16,9 +16,9 @@
 #include "shell-window-tracker-private.h"
 #include "shell-app-system.h"
 #include "shell-app-private.h"
-#include "st/st-texture-cache.h"
 #include "shell-global.h"
 #include "shell-marshal.h"
+#include "st.h"
 
 #include "display.h"
 #include "window.h"
@@ -941,7 +941,7 @@ shell_startup_sequence_create_icon (ShellStartupSequence *sequence, guint size)
 
   themed = g_themed_icon_new (icon_name);
   texture = st_texture_cache_load_gicon (st_texture_cache_get_default (),
-                                         themed, size);
+                                         NULL, themed, size);
   g_object_unref (G_OBJECT (themed));
   return texture;
 }
